!-------------------------------------------------------------------------------

!                      Code_Saturne version 5.0.8
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! --------

!> \file cs_user_fluid_structure_interaction.f90
!>
!> \brief User subroutines dedicated to Fluid-Structure interaction modeling
!>
!>   \par Management of internal Fluid-Structure coupled calculations:
!>        a simplified solid model is used
!>        (linear "mass, friction and spring" modeling).
!>        Here are 2 differents subroutines that need to be filled :
!>
!>     - \ref usstr1 : Called at the beginning of the calculation. It enables
!>                     one to define internal structures and corresponding
!>                     initial conditions (initial displacement and velocity).
!>
!>     - \ref usstr2 : Called at each time step of the calculation. Here one
!>                     structural parameters
!>                     (considered to be potentially time dependent),
!>                     i.e. Mass, Friction, Stiffness and Fluid Stresses.
!>
!>   \par Fluid-Structure coupling with Code_Aster:
!>        the user subroutine \ref usaste has to be used.
!>
!>   \par Examples of data settings for fluid-structure interaction (FSI):
!>        Several examples are available
!>        \ref cs_user_fluid_structure_interaction "here".
!
!-------------------------------------------------------------------------------

!===============================================================================
! Function :
! ----------

!> \brief Definition of internal structures and corresponding initial conditions
!>       (initial displacement and velocity ).
!
!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]    idfstr         boundary faces -> structure definition
!> \param[in]    aexxst,bexxst  prediction coefficients of structural data
!> \param[in]    cfopre         prediction coefficients of fluid forces
!> \param[in]    xstr0          initial displacement of internal structures
!> \param[in]    vstr0          initial velocity of internal structures
!> \param[in]    xstreq         displacement of initial mesh compared to
!>                              structures position at equilibrium
!______________________________________________________________________________!

subroutine usstr1 &
 ( idfstr ,                                                       &
   aexxst , bexxst , cfopre ,                                     &
   xstr0  , vstr0  , xstreq )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use optcal
use entsor
use albase
use parall
use period
use mesh

!===============================================================================

implicit none

! Arguments

integer          idfstr(nfabor)

double precision aexxst, bexxst, cfopre
double precision xstr0(3,nstrmx), xstreq(3,nstrmx)
double precision vstr0(3,nstrmx)

! Local variables

double precision hn_FS , heq_FS
double precision L_FS , D_FS

integer istr , ifac , ilelt , nlelt
integer i_inlet 
integer iph , cor_index , sep_cor

double precision delta0_FS, deltaa_FS, P_FS , depl_FS , depl_FS_2 , depl_FS_2_10 , a_FS , a_FS_2
double precision depl_mid , depl_mn
double precision k

! Parameters for imposting BCs within 1D Mesh over the core
!----------------------------------------------------------
double precision M_star_FS , N_star_FS
double precision x_star_FS , y_star_FS , z_star_FS , x_FS , y_FS

integer inod , counter

double precision x_00_FS , y_00_FS , x_01_FS , y_01_FS
double precision x_10_FS , y_10_FS , x_11_FS , y_11_FS

double precision d_Ofac_FS
double precision d_00_FS , d_01_FS , d_10_FS , d_11_FS
double precision d_min_FS
double precision eps_FS

integer M_0_FS , N_0_FS , M_1_FS , N_1_FS 

integer M_FS , N_FS

double precision theta_minus , theta_plus , theta_mid , theta_FS
integer i_minus , i_plus

integer n_cor
!----------------------------------------------------------

double precision , allocatable , save :: coord_0_FS(:,:)

integer, allocatable, dimension(:) :: lstelt

!----------------------------------------------------------

double precision m_lin, k_lin , f_FS , w_FS

!----------------------------------------------------------

integer counter_BC

!----------------------------------------------------------

integer i_P2C

!----------------------------------------------------------

!===============================================================================

! ****************************************************************************
! write(nfecra,*) 'DEBUGGING: ENTERING THE SUBROUTINE USSTR1'
! ****************************************************************************

! Additional geometrical parameter (hexagon edge)
hn_FS  = 3.0d-3
heq_FS = hn_FS

! Hexagon edge length
L_FS = 0.07332d0

! Initial position with respect to the nominal one
! depl_FS = -heq_FS/2.d0

! Initial expansion coefficient
! a_FS   = -0.5d0  
a_FS   = 0d0

! a_FS_2   = -0.816d0
a_FS_2   = 0.816d0
! a_FS_2   = 0.333d0
! a_FS_2   = -5.d-2
! a_FS_2   = -5.d-1
! a_FS_2   = 5.d-1
! a_FS_2   = 5.d-2

#if 0
! Parameters for initial velocity
m_lin = 85.22d-6 !*1.d6 ! MUST CORRESPOND TO THE STRUCTURE PARAMATER hexa_m_lin !
k_lin = 2.526d-2 !*1.d6 ! MUST CORRESPOND TO THE STRUCTURE PARAMATER hexa_k_lin !
f_FS = sqrt( k_lin / m_lin )
w_FS = 2.d0 * pi * f_FS
#endif 

! Initial gap
delta0_FS  = hn_FS*(1 + a_FS)

! Initial position with respect to the nominal one
! depl_FS = heq_FS/2.d0
depl_FS = -hn_FS*a_FS
depl_FS_2_10 = hn_FS*a_FS_2

! Bundle step
P_FS = sqrt(3.d0)*L_FS + delta0_FS

! Tolerance
eps_FS = 1.0d-5

!===============================================================================
! 1.  Parameters of Newmark HHT algorithm
!===============================================================================
! aexxst = 0.5d0
! aexxst = 1d0

! bexxst = 1d0
! bexxst = 0d0

! cfopre = 1.d0
! cfopre = 1d-2
! cfopre = 1.d0/50.d0

!===============================================================================
! 2.  Initialization
!===============================================================================

! Allocate a temporary array for boundary faces selection
allocate(lstelt(nfabor))

!===============================================================================
! 3.  Management of normal faces
!===============================================================================

call getfbr('normal[0,0,1,1e-4] or normal[0,0,-1,1e-4]', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
  idfstr(ifac) = 0
  
enddo

!===============================================================================
! 4.  Definition of internal structures and initial displacement
!===============================================================================

call getfbr('not(normal[0,0,1,1e-4] or normal[0,0,-1,1e-4])', nlelt, lstelt)

! ---------------------------------------------------------
! Allocate an array to store the initial coordinates values
! ---------------------------------------------------------
if (ntcabs .eq. 0 .and. (.not. allocated(coord_0_FS))) then 
  allocate(coord_0_FS(3,nfabor))
endif
! ---------------------------------------------------------

counter_BC = 0

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
  
 ! --------------------------------------
 ! Use the inital coordinates at any time
 ! --------------------------------------
  if (ntcabs .eq. 0) then
  
    x_star_FS = cdgfbo(1,ifac)
    y_star_FS = cdgfbo(2,ifac)
    z_star_FS = cdgfbo(3,ifac)
  
    coord_0_FS(1,ifac) = x_star_FS
    coord_0_FS(2,ifac) = y_star_FS
    coord_0_FS(3,ifac) = z_star_FS
    
  else
  
    x_star_FS = coord_0_FS(1,ifac)
    y_star_FS = coord_0_FS(2,ifac)
    z_star_FS = coord_0_FS(3,ifac)
    
  endif
 ! --------------------------------------
 
 ! First step: calculation of the (M_star, N_star) coordinates
  M_star_FS = 1.d0/P_FS * (1.d0/sqrt(3.d0)*x_star_FS - y_star_FS)
  N_star_FS = 1.d0/P_FS * (1.d0/sqrt(3.d0)*x_star_FS + y_star_FS)
  
  ! Second step: get the nearest integer coordinates (M,N)
  M_0_FS = floor(M_star_FS)
  M_1_FS = floor(M_star_FS) + 1
  
  N_0_FS = floor(N_star_FS)
  N_1_FS = floor(N_star_FS) + 1

#if 0  
  ! #########################################################################################
  ! Debugging step
  ! #########################################################################################
  open(unit=30, file="/volatile/sh248391/Saturne/CORONAS_1D_IFS/RESU/Debugging_IFS.dat", status="unknown")
  ! open(unit=19, file="/export/home/sh248391/Saturne/CORONAS_1D_MESH/RESU/Debugging.dat", status="unknown")
  write (30,'((e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(I3.2),(A),(I3.2),(A),(I3.2),(A),(I3.2))') &
  x_star_FS , ',' , y_star_FS , ',' , M_star_FS , ',' , N_star_FS , ',' , M_0_FS , ',' , M_1_FS , ',' , N_0_FS , ',' , N_1_FS
  ! #########################################################################################
#endif  

! Third step: calculation of the hexagon (M,N) center coordinates
  x_00_FS = sqrt(3.d0)/2.d0 * P_FS * (M_0_FS + N_0_FS)
  y_00_FS = 1/2.d0          * P_FS * (N_0_FS - M_0_FS)
  
  x_01_FS = sqrt(3.d0)/2.d0 * P_FS * (M_0_FS + N_1_FS)
  y_01_FS = 1/2.d0          * P_FS * (N_1_FS - M_0_FS)
  
  x_10_FS = sqrt(3.d0)/2.d0 * P_FS * (M_1_FS + N_0_FS)
  y_10_FS = 1/2.d0          * P_FS * (N_0_FS - M_1_FS)
  
  x_11_FS = sqrt(3.d0)/2.d0 * P_FS * (M_1_FS + N_1_FS)
  y_11_FS = 1/2.d0          * P_FS * (N_1_FS - M_1_FS)
  
  ! Distance to the center of nearest hexagon
  d_00_FS = sqrt((x_star_FS - x_00_FS)**(2.d0) + (y_star_FS - y_00_FS)**(2.d0))
  d_01_FS = sqrt((x_star_FS - x_01_FS)**(2.d0) + (y_star_FS - y_01_FS)**(2.d0))
  d_10_FS = sqrt((x_star_FS - x_10_FS)**(2.d0) + (y_star_FS - y_10_FS)**(2.d0))
  d_11_FS = sqrt((x_star_FS - x_11_FS)**(2.d0) + (y_star_FS - y_11_FS)**(2.d0))
  
  ! Test of presence of the face
  ! Application of the corresponding BC according to the test crterion
  
  ! Initialization
  i_inlet = 0

  ! ------------------------------------------------------------------
  d_min_FS = min(d_00_FS , d_01_FS , d_10_FS , d_11_FS)
  ! ------------------------------------------------------------------

  ! ------------------------------------------------------------------
  if ((d_00_FS .ge. (d_min_FS - eps_FS)) .and. (d_00_FS .le. (d_min_FS + eps_FS))) then
    M_FS = M_0_FS
    N_FS = N_0_FS   
    i_inlet = i_inlet + 1
  endif    
        
  if ((d_01_FS .ge. (d_min_FS - eps_FS)) .and. (d_01_FS .le. (d_min_FS + eps_FS))) then
    M_FS = M_0_FS
    N_FS = N_1_FS    
    i_inlet = i_inlet + 1
  endif  

  if ((d_10_FS .ge. (d_min_FS - eps_FS)) .and. (d_10_FS .le. (d_min_FS + eps_FS))) then
    M_FS = M_1_FS
    N_FS = N_0_FS   
    i_inlet = i_inlet + 1
  endif

  if ((d_11_FS .ge. (d_min_FS - eps_FS)) .and. (d_11_FS .le. (d_min_FS + eps_FS))) then
    M_FS = M_1_FS
    N_FS = N_1_FS   
    i_inlet = i_inlet + 1
  endif
  ! ------------------------------------------------------------------

#if 0  
  ! #########################################################################################
  ! Debugging step
  ! #########################################################################################
  open(unit=31, file="/volatile/sh248391/Saturne/CORONAS_1D_IFS/RESU/Distances_IFS.dat", status="unknown")
  ! open(unit=20, file="/export/home/sh248391/Saturne/CORONAS_1D_MESH/RESU/Distances.dat", status="unknown")
  write (31,'((e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(i1.1))') &
  x_star_FS , ',' , y_star_FS , ',' , d_00_FS , ',' , d_01_FS , ',' , d_10_FS , ',' , d_11_FS ,&
   ',' , d_min_FS , ',' , i_inlet
  ! #########################################################################################
#endif  
  
  ! ------------------------------------------------------------------  
  x_FS = sqrt(3.d0)/2.d0 * P_FS * (M_FS + N_FS)
  y_FS = 1/2.d0 * P_FS * (N_FS - M_FS)
  ! ------------------------------------------------------------------

#if 0  
  ! #########################################################################################
  ! Debugging step
  ! #########################################################################################
  open(unit=32, file="/volatile/sh248391/Saturne/CORONAS_1D_IFS/RESU/Hexagones_IFS.dat", status="unknown")
  ! open(unit=19, file="/export/home/sh248391/Saturne/CORONAS_1D_MESH/RESU/Debugging.dat", status="unknown")
  write (32,'((e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(I3.2),(A),(I3.2))') &
  x_star_FS , ',' , y_star_FS , ',' , x_FS , ',' , y_FS , ',' , M_FS , ',' , N_FS
  ! #########################################################################################
#endif  

  ! *****************
  ! Number of coronas
  ! *****************
  n_cor = 1
  ! *****************
  
  ! ------------------------------------------------------------------
  ! Structures numbering
  ! ------------------------------------------------------------------    
  if (i_inlet .eq. 1) then
    
    if (M_FS .lt. 0) then
      idfstr(ifac) = 1 - (n_cor-M_FS)*(M_FS+n_cor+1)/2 + ((M_FS+n_cor)*(2*n_cor+1) + (N_FS+n_cor))   
    
    else if (M_FS .gt. 1) then
      idfstr(ifac) = 1 - n_cor*(n_cor+1)/2 - M_FS*(M_FS-1)/2 + ((M_FS+n_cor)*(2*n_cor+1) + (N_FS+n_cor))

    ! :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::;;;;
    ! PISE-2C: The central hexagon is fixed on the base and then assumed to be motionless
    ! :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#if 0    
    else if (M_FS .eq. 0 .and. N_FS .eq. 0) then
       idfstr(ifac) = 0
#endif    
    ! ::::::::::::::::::::::::::::::::::::::::::::::::::::
          
    else
      idfstr(ifac) = 1 - n_cor*(n_cor+1)/2 + ((M_FS+n_cor)*(2*n_cor+1) + (N_FS+n_cor))   
      
    endif
                      
  else if (i_inlet .eq. 2) then
    
    idfstr(ifac) = 0
          
  endif
  ! ------------------------------------------------------------------
    
  istr = idfstr(ifac)

#if 0
  ! ------------------------------------------------------------------    
  ! Phénix position numbering (20,20) at the center
  ! ------------------------------------------------------------------
  iph = 0
  if (iph .eq. 1) then   
    M_FS = 20 + M_FS
    N_FS = 20 + N_FS
  endif 
  ! ------------------------------------------------------------------
#endif 
  
#if 0  
  ! #########################################################################################
  ! Debugging step
  ! #########################################################################################
  open(unit=32, file="/volatile/sh248391/Saturne/CORONAS_1D_IFS/RESU/Hexagones_IFS.dat", status="unknown")
  ! open(unit=19, file="/export/home/sh248391/Saturne/CORONAS_1D_MESH/RESU/Debugging.dat", status="unknown")
  write (32, &
  '((e13.6),(A),(e13.6),(A),(e13.6),(A),(e13.6),(A),(I3.2),(A),(I3.2),(A),(I3.2),(A),(I3.2),(A),(I4.3),(A),(I4.3))') & 
  x_star_FS , ',' , y_star_FS , ',' , x_FS , ',' , y_FS , ',' , M_FS , ',' , N_FS , ',' , i_inlet , ',' , istr &
  , ',' , ilelt , ',' , nlelt
  ! #########################################################################################
#endif  
  
  ! -------------------------------------------------
  ! The central hexagon stays at its nominal position
  ! -------------------------------------------------
  if ((M_FS .eq. 0) .and. (N_FS .eq. 0)) then

!#if 0  
    xstreq(1,istr) = 0.d0
    xstreq(2,istr) = 0.d0
    xstreq(3,istr) = 0.d0
!#if 0      
    !xstr0(1,istr) = 0.d0
    !xstr0(2,istr) = 0.d0
    !xstr0(3,istr) = 0.d0
    
    !vstr0(1,istr) = 0.d0
    !vstr0(2,istr) = 0.d0
    !vstr0(3,istr) = 0.d0
!#endif
!#endif

#if 0  
    xstreq(1,istr) = x_star_FS
    xstreq(2,istr) = y_star_FS
    xstreq(3,istr) = z_star_FS
    
    xstr0(1,istr) = x_star_FS
    xstr0(2,istr) = y_star_FS
    xstr0(3,istr) = z_star_FS
    
    vstr0(1,istr) = 0.d0
    vstr0(2,istr) = 0.d0
    vstr0(3,istr) = 0.d0
#endif
        
  ! --------------------------------------------------
  
  ! ---------------------------------------------
  ! The others are initially pushed to the center
  ! -> The core is initially compacted
  ! --------------------------------------------- 
  else
    
    if (i_inlet .eq. 1) then
         
    ! Azimutal of the (M,N) hexagon
    if ((x_FS .ge. -eps_FS) .and. (x_FS .le. eps_FS)) then
      theta_FS = (abs(y_FS)/y_FS)/2.d0*pi
    else
      theta_FS = atan(y_FS/x_FS) + (1-(abs(x_FS)/x_FS))/2.d0*pi
    endif

  ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  ! 04/09/2019 - Generalization of the outwards movement
  ! From an homogeneous movement to a non-homogeneous one
  ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  ! ************************************************************************
  ! 1 - Homogeneous structure movement
  ! ************************************************************************
  ! depl_FS_2 = depl_FS_2_10
  ! ************************************************************************
   
  ! ************************************************************************
  ! 2 - Modification of the displacement according to the ring number
  ! Handling this structure displacement field by means of the (M,N) 
  ! coordinates depending of the angular sector
  ! ************************************************************************
!#if 0  
  ! Displacement modulation coefficient
  k = 1
  
  ! Corona index to seperate the two kind of structures displacement
  sep_cor = 1
  
  ! -----------------------------------------------------------------
  ! Sector 1
  ! -----------------------------------------------------------------
  if ( (theta_FS .ge. -pi/2.d0) .and. (theta_FS .lt. -pi/6.d0)) then
    cor_index = M_FS
  ! -----------------------------------------------------------------
  ! -----------------------------------------------------------------
  ! Sector 2
  ! -----------------------------------------------------------------  
  else if ((theta_FS .ge. -pi/6.d0) .and. (theta_FS .lt. pi/6.d0)) then
    cor_index = M_FS + N_FS
  ! -----------------------------------------------------------------
  ! -----------------------------------------------------------------
  ! Sector 3
  ! -----------------------------------------------------------------       
  else if ((theta_FS .ge. pi/6.d0) .and. (theta_FS .lt. pi/2.d0)) then
    cor_index = N_FS
  ! -----------------------------------------------------------------
  ! -----------------------------------------------------------------
  ! Sector 4
  ! ----------------------------------------------------------------- 
  else if ((theta_FS .ge. pi/2.d0) .and. (theta_FS .lt. 5.d0*pi/6.d0)) then
    cor_index = -M_FS
  ! -----------------------------------------------------------------
  ! -----------------------------------------------------------------
  ! Sector 5
  ! ----------------------------------------------------------------- 
  else if ((theta_FS .ge. 5.d0*pi/6.d0) .and. (theta_FS .lt. 7.d0*pi/6.d0)) then
    cor_index = -(M_FS+N_FS)
  ! -----------------------------------------------------------------
  ! -----------------------------------------------------------------
  ! Sector 6
  ! -----------------------------------------------------------------
  else if ((theta_FS .ge. 7.d0*pi/6.d0) .and. (theta_FS .lt. 3.d0*pi/2.d0)) then
    cor_index = -N_FS
  ! ----------------------------------------------------------------- 
  endif

  if (cor_index .le. sep_cor) then
    depl_FS_2 = k*cor_index*depl_FS_2_10
    ! depl_FS_2 = 0d0      
  else
    depl_FS_2 = k*sep_cor*depl_FS_2_10
    ! depl_FS_2 = 0d0    
  endif
  
!#endif  
  ! ************************************************************************

  ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
    i_minus = floor(3.d0/2.d0 + theta_FS/(pi/3.d0))
    i_plus  = floor(3.d0/2.d0 + theta_FS/(pi/3.d0)) + 1
            
    theta_minus = (2.d0 * i_minus - 3.d0)*pi/6.d0
    theta_plus  = (2.d0 * i_plus  - 3.d0)*pi/6.d0
            
    theta_mid = (theta_minus + theta_plus)/2.d0
    depl_mid  = depl_FS_2*cos(pi/6.d0)
    ! depl_mid  = depl_FS*cos(pi/6.d0)
            
    depl_mn = depl_mid/cos(theta_FS - theta_mid)
     
!#if 0
    xstreq(1,istr) = 0.d0
    xstreq(2,istr) = 0.d0
    xstreq(3,istr) = 0.d0
    
    ! xstreq(1,istr) = depl_mn*cos(theta_FS)
    ! xstreq(2,istr) = depl_mn*sin(theta_FS)
    ! xstreq(3,istr) = 0.d0

    ! xstreq(1,istr) = -hn_FS*abs(a_FS_2)*cos(theta_FS)
    ! xstreq(2,istr) = -hn_FS*abs(a_FS_2)*sin(theta_FS)
    ! xstreq(3,istr) = 0.d0
      
    ! xstr0(1,istr) = hn_FS*a_FS_2*cos(theta_FS)
    ! xstr0(2,istr) = hn_FS*a_FS_2*sin(theta_FS)
    ! xstr0(3,istr) = 0.d0

    xstr0(1,istr) = depl_mn*cos(theta_FS)
    xstr0(2,istr) = depl_mn*sin(theta_FS)
    xstr0(3,istr) = 0.d0
        
    vstr0(1,istr) = 0.d0
    vstr0(2,istr) = 0.d0
    vstr0(3,istr) = 0.d0
!#endif

#if 0    
    xstreq(1,istr) = x_star_FS - depl_mn*cos(theta_FS)
    xstreq(2,istr) = y_star_FS - depl_mn*sin(theta_FS)
    xstreq(3,istr) = z_star_FS
    
    xstr0(1,istr) = x_star_FS
    xstr0(2,istr) = y_star_FS
    xstr0(3,istr) = z_star_FS
    
    vstr0(1,istr) = 0.d0
    vstr0(2,istr) = 0.d0
    vstr0(3,istr) = 0.d0
#endif
            
  endif
  
  endif
  ! ---------------------------------------------
       
enddo

  ! close(29)
  ! close(30)
  ! close(31)
  ! close(32)  
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! Deallocate the temporary array
deallocate(lstelt)
! if (ntcabs .eq. ntmabs) then 
!   deallocate(coord_0_FS)
! endif

! ****************************************************************************
! write(nfecra,*) 'DEBUGGING: EXITING THE SUBROUTINE USSTR1'
! ****************************************************************************

return
end subroutine usstr1


!===============================================================================
! Purpose :
! ---------

!> \brief Definition of structural parameters in case of Fluid Structure
!>        internal coupling : Mass, Friction, Stiffness anf Fluid Stresses.
!
!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]    nbstru         nombre de structures definies
!> \param[in]    idfstr         definition des structures
!> \param[in]    dtcel          time step (per cell
!> \param[in]    xmstru         matrix of structural mass
!> \param[out]   xcstru         matrix of structural friction
!> \param[out]   xkstru         matrix of structural stiffness
!> \param[in]    xstreq         displacement of initial mesh compared to
!> \param[in]    xstr           structural displacement
!> \param[in]    vstr           structural velocity
!> \param[in]    forstr         forces acting on structures (take forces)
!> \param[out]   dtstr          structural time step
!______________________________________________________________________________!

subroutine usstr2 &
 ( nbstru ,                                                       &
   idfstr ,                                                       &
   dtcel  ,                                                       &
   xmstru , xcstru , xkstru , xstreq , xstr   , vstr   , forstr , &
   dtstr  )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use optcal
use albase
use parall
use period
use mesh

use numvar
use field

!===============================================================================

implicit none

! Arguments

integer          nbstru

integer          idfstr(nfabor)

double precision dtcel(ncelet)
double precision xmstru(3,3,nstrmx)
double precision xcstru(3,3,nstrmx)
double precision xkstru(3,3,nstrmx)
double precision xstreq(3,nstrmx)
double precision xstr(3,nstrmx)
double precision vstr(3,nstrmx)
double precision forstr(3,nstrmx)
double precision dtstr(nstrmx)

! Local variables

integer          ii, jj, istr, iel, ifac
double precision hexa_m_lin , hexa_c_lin , hexa_k_lin
double precision hexa_m_lin_P2C(19) , hexa_c_lin_P2C(19) , hexa_k_lin_P2C(19)
double precision, dimension(:), pointer :: cvar_pre_BC

!----------------------------------------------------------

integer i_P2C
double precision f_P2C, H_P2C , deltaZ , reducFac
!----------------------------------------------------------

!===============================================================================

! ****************************************************************************
! write(nfecra,*) 'DEBUGGING: ENTERING THE SUBROUTINE USSTR2'
! ****************************************************************************

call field_get_val_s(ivarfl(ipr), cvar_pre_BC)

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Definition of the structures' parameters
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! ##############################################
! /!\ Hexagon mass within a height of 1e-6 m !!!
! => Multiplication of the linear mass by 1e-6 !
! ##############################################

! ********************************************************************
! 1 - Structure parameters for the first vibration mode at f = 2.74 Hz
! ********************************************************************
#if 0
hexa_m_lin = 85.22d-6!*1.d6
hexa_c_lin = 0.d0
hexa_k_lin = 2.526d-2!*1.d6 
#endif
! ********************************************************************

! ********************************************************************
! 2 - Structure parameters for the second vibration mode at f = 20 Hz
! ********************************************************************
!#if 0
hexa_m_lin = 85.22d-6!*1.d6
hexa_c_lin = 0.d0
hexa_k_lin = 1.346d0!*1.d6
!#endif
! ********************************************************************

! ********************************************************************
! 3 - Structure parameters for the second vibration mode at f = 60 Hz
! ********************************************************************
#if 0
hexa_m_lin = 85.22d-6!*1.d6
hexa_c_lin = 0.d0
hexa_k_lin = 1.211d1!*1.d6
#endif
! ********************************************************************

! ********************************************************************
! 4 - Structure parameters at f = 91 Hz (to reach Psat within CN1)
! ********************************************************************
#if 0
hexa_m_lin = 85.22d-6!*1.d6
hexa_c_lin = 0.d0
hexa_k_lin = 2.526d1!*1.d6 ! f = 91 Hz
#endif
  ! ********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!===============================================================================
! Definition of structural parameters: mass, friction, stiffness and stresses.
!===============================================================================

! --- Matrices xmstru, xcstru and xkstru are initialized to the value of 0.
do istr = 1, nbstru

  do ii = 1, 3
    do jj = 1, 3
      xmstru(ii,jj,istr) = 0.d0
      xcstru(ii,jj,istr) = 0.d0
      xkstru(ii,jj,istr) = 0.d0
    enddo
  enddo

enddo

! ----------------------------------
! Isotropic modeling
! ----------------------------------
do istr = 1, nbstru

  do ii = 1, 3
      xmstru(ii,ii,istr) = hexa_m_lin
      xcstru(ii,ii,istr) = hexa_c_lin
      xkstru(ii,ii,istr) = hexa_k_lin 
  enddo
  
enddo
! ----------------------------------

! ****************************************************************************
! write(nfecra,*) 'DEBUGGING: EXITING THE SUBROUTINE USSTR2'
! ****************************************************************************

return

end subroutine usstr2

!===============================================================================
