!-------------------------------------------------------------------------------

!                      Code_Saturne version 6.0.1
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2019 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================

!> \file cs_user_parameters.f90
!>
!> \brief User subroutines for input of calculation parameters (Fortran modules).
!>        These subroutines are called in all cases.
!>
!>  See \subpage f_parameters for examples.
!>
!>   If the Code_Saturne GUI is used, this file is not required (but may be
!>   used to override parameters entered through the GUI, and to set
!>   parameters not accessible through the GUI).
!>
!>   Several routines are present in the file, each destined to defined
!>   specific parameters.
!>
!>   To modify the default value of parameters which do not appear in the
!>   examples provided, code should be placed as follows:
!>   - usipsu   for numerical and physical options
!>   - usipes   for input-output related options
!>
!>   As a convention, "specific physics" defers to the following modules only:
!>   pulverized coal, gas combustion, electric arcs.
!>
!>   In addition, specific routines are provided for the definition of some
!>   "specific physics" options.
!>   These routines are described at the end of this file and will be activated
!>   when the corresponding option is selected in the usppmo routine.
!-------------------------------------------------------------------------------

!===============================================================================

!> \brief User subroutine for selection of specific physics module

!> Define the use of a specific physics amongst the following:
!>   - combustion with gas / coal / heavy fuel oil
!>   - compressible flows
!>   - electric arcs
!>   - atmospheric modelling
!>   - radiative transfer
!>   - cooling towers modelling
!>
!>    Only one specific physics module can be activated at once.

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     ixmlpu        indicates if the XML file from the GUI is used
!>                              (1 : yes, 0 : no)
!______________________________________________________________________________!

subroutine usppmo &
 ( ixmlpu )

!===============================================================================
! Module files
!===============================================================================

use paramx
use entsor
use cstphy
use ppppar
use ppthch
use ppincl
use ppcpfu
use coincl
use radiat
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer ixmlpu

return
end subroutine usppmo


!===============================================================================

!> \brief User subroutine for input of model selection parameters.

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]      ixmlpu       indicates if the XML file from the GUI is used
!>                              used (1: yes, 0: no
!> \param[in, out] iturb        turbulence model
!> \param[in, out] itherm       thermal model
!> \param[in, out] iale         ALE module
!______________________________________________________________________________!

subroutine usipph &
 ( ixmlpu, iturb , itherm, iale )

!===============================================================================
! Module files
!===============================================================================

use entsor, only: nfecra ! No other module should appear here
use optcal, only: irijco ! No other module should appear here

!===============================================================================

implicit none

! Arguments

integer ixmlpu
integer iturb, itherm, iale

! Local variables

!===============================================================================

!>    In this subroutine, only the parameters which already appear may
!>    be set, to the exclusion of any other.
!>
!>    If we are not using the Code_Saturne GUI:
!>    All the parameters which appear in this subroutine must be set.
!>
!>    If we are using the Code_Saturne GUI:
!>    parameters protected by a test of the form:
!>
!>      if (ixmlpu.eq.0) then
!>         ...
!>      endif
!>
!>    should already have been defined using the GUI, so only
!>    experts should consider removing the test and adapting them here.

!===============================================================================

! Workspace using input files (no GUI)
ixmlpu =  0

! No turbulence modelling is used -> for a relative medium-high Reynolds number,
! the refined mesh which is used is eventually expected to simulate a certain part
! of the Kolmogorov spectrum (hypothesis)
iturb =   0

! No effect of temperature is taken into account
itherm =  0

! Activation of ALE (Arbitrary Lagrangian Eulerian) method
! iale =    1 ! Mouvement inter-plaques
iale =    1

!----
! Formats
!----

return
end subroutine usipph


!===============================================================================

!> \brief User subroutine for the input of additional user parameters.
!
!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nmodpp         number of active specific physics models
!______________________________________________________________________________!

subroutine usipsu &
 ( nmodpp )

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use dimens
use numvar
use optcal
use cstphy
use entsor
use parall
use period
use ihmpre
use albase
use ppppar
use ppthch
use ppincl
use coincl
use cpincl
use field
use cavitation
use post
use rotation
use cs_c_bindings

use alstru

!===============================================================================

implicit none

! Arguments

integer nmodpp

!===============================================================================

!>  This subroutine allows setting parameters
!>  which do not already appear in the other subroutines of this file.
!>
!>  It is possible to add or remove parameters.
!>  The number of physical properties and variables is known here.

!===============================================================================

ro0 =     1.0d3
viscl0 =  1.0d-3
p0 =      1.0d5

! ischtp = 2

! dtref = 1.d-5
! dtref = 1.d-4
dtref = 1.d-3
! dtref = 5.d-3
! dtref = 1.d-2

ntmabs = 1

italin = 1

nalimx = 1
! nalimx = 5
! nalimx = 5d1
! nalimx = 2d2
! nalimx = 2d3
! nalimx = 2d4

! epalim = 1.d-3
! epalim = 1.d-5
epalim = 1.d-8

!----
! Formats
!----

return
end subroutine usipsu

!===============================================================================

!> \brief User subroutine for the input of additional user parameters for
!>        input/output.

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nmodpp       number of active specific physics models
!______________________________________________________________________________!

subroutine usipes &
 ( nmodpp )

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use dimens
use numvar
use optcal
use cstphy
use entsor
use field
use parall
use period
use ihmpre
use post
use ppppar
use ppthch
use ppincl
use cs_c_bindings

use albase, only: fdiale

!===============================================================================

implicit none

! Arguments

integer nmodpp

!===============================================================================

!>     This subroutine allows setting parameters
!>     which do not already appear in the other subroutines of this file.
!>
!>     It is possible to add or remove parameters.
!>     The number of physical properties and variables is known here.

!===============================================================================

! 06/08/18 - Local variables for fields post-processing

integer ii
integer f_id, idim1, ifllog

type(var_cal_opt) :: vcopt

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Post-processing
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! Post-processing parameters

! Writting time step of the outputs
! -> every 10 times steps
! emphis = "$SAT/RESU/postprocessing"
! nthist = 10
! frhist = 50
! tplfmt = 1

! call post_activate_by_time_step ()

! Mesh deformation post-processing over time
ihistr  = 1
! call cs_post_set_deformable ()
 
! Post-processing of varables and properties -> Pressure field
! 1 to activate postprocessing output, 0 to deactivate
! f_id = ivarfl(ipr)
! call field_set_key_int(f_id, keyvis, 1)

! Yplus
f_id = iprpfl(iyplbr)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Courant-Friedrich Number (CFL)
f_id = iprpfl(icour)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Fourier number
f_id = iprpfl(ifour)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Laminar viscosity
f_id = iprpfl(iviscl)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Laminar viscosity
f_id = iprpfl(ivisma)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Total pressure
f_id = iprpfl(iprtot)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0) 

! Mesh displacement
call field_set_key_int(fdiale, keyvis, 0)
call field_set_key_int(fdiale, keylog, 0) 

! call cs_user_postprocess_activate(ntmabs , ntcabs, ttcabs)

return
end subroutine usipes


!===============================================================================

