/*============================================================================
 * Define postprocessing output.
 *============================================================================*/

/* Code_Saturne version 5.0.8 */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2018 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include "stdlib.h"
#include "string.h"

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_geom.h"
#include "cs_interpolate.h"
#include "cs_mesh.h"
#include "cs_selector.h"
#include "cs_parall.h"
#include "cs_post.h"
#include "cs_post_util.h"
#include "cs_probe.h"
#include "cs_time_plot.h"

#include "cs_field_pointer.h"
#include "cs_parameters.h"
#include "cs_physical_constants.h"
#include "cs_turbulence_model.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define post-processing writers.
 *
 * The default output format and frequency may be configured, and additional
 * post-processing writers allowing outputs in different formats or with
 * different format options and output frequency than the main writer may
 * be defined.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_postprocess_writers(void)
{
  /* Set time plot file writer flush behavior defaults. */

  /*! [post_set_tp_flush] */
  cs_time_plot_set_flush_default(1800, /* flush_wtime */
                                 -1);  /* n_buffer_steps */
  /*! [post_set_tp_flush] */

  /* Default output format and options */

  /* Redefine default writer */
  /* ----------------------- */

  /*! [post_define_writer_m1] */
  cs_post_define_writer(CS_POST_WRITER_DEFAULT,       /* writer_id */
                        "results",                    /* writer name */
                        "postprocessing",             /* directory name */
                        "MED",                        /* format_name */
                        "",                           /* format_options */
                        FVM_WRITER_TRANSIENT_COORDS,
                        true,                        /* output_at_start */
                        true,                        /* output_at_end */
                        1e1,                         /* frequency_n */
                        -1);                         /* frequency_t */
  
  /* Writer to re-define parameters of probes */
  /*
  cs_post_define_writer(CS_POST_WRITER_PROBES,      
                        "",                   
                        "monitoring",             
                        "time_plot",                     
                        "csv",                           
                        FVM_WRITER_TRANSIENT_COORDS,
                        false,                        
                        false,                        
                        10,                           
                        0.0001);
   */ 

}



/*----------------------------------------------------------------------------*/
/*!
 * \brief Define monitoring probes and profiles.
 *
 * Profiles are defined as sets of probes.
 */
/*----------------------------------------------------------------------------*/

/*#if 0*/
void 
cs_user_postprocess_probes(void)
{
  /* Define monitoring probes */

  /* A writer (id = CS_POST_WRITER_PROBES) using the format "time_plot" is
     associated by default to a set of monitoring probes.
     This is not the case for a profile. */
     
  {
  
  /* ################################################################################ */
  /* ### Probes position to copy/paste into Code_Saturne cs_user_postprocess file ### */
  /* ################################################################################ */
  
  /* # PAS DES SONDES INTRIEURES (Ox) :0.146640E-02
  /* # N CELLS (Ox) :50
  /* # Y SONDES (Ox) :-.6425E-01

  /* # DEMI-ENTREPLAT : 0.634970E-01

  /* ################################################################################ */
  /* ################################################################################ */

  /* ************************************ */
  /* Creation of the set of SINGLE probes */
  /* ************************************ */

  /* Probes along the (Ox) axis at progressive abscissa - y = 0 */

  cs_probe_set_t  *xset_0 = cs_probe_set_create("x_axis_probes_y0");
  /* cs_probe_set_option(xset_0 , "transient_location" , "true");

  /* Segment of probes */
  cs_probe_set_add_probe(xset_0 , -0.366600E-01 , -0.64246983E-01 , 0.5E-06 , "X00Y0" );
  cs_probe_set_add_probe(xset_0 , -0.359268E-01 , -0.64246983E-01 , 0.5E-06 , "X01Y0" );
  cs_probe_set_add_probe(xset_0 , -0.344604E-01 , -0.64246983E-01 , 0.5E-06 , "X02Y0" );
  cs_probe_set_add_probe(xset_0 , -0.329940E-01 , -0.64246983E-01 , 0.5E-06 , "X03Y0" );
  cs_probe_set_add_probe(xset_0 , -0.315276E-01 , -0.64246983E-01 , 0.5E-06 , "X04Y0" );
  cs_probe_set_add_probe(xset_0 , -0.300612E-01 , -0.64246983E-01 , 0.5E-06 , "X05Y0" );
  cs_probe_set_add_probe(xset_0 , -0.285948E-01 , -0.64246983E-01 , 0.5E-06 , "X06Y0" );
  cs_probe_set_add_probe(xset_0 , -0.271284E-01 , -0.64246983E-01 , 0.5E-06 , "X07Y0" );
  cs_probe_set_add_probe(xset_0 , -0.256620E-01 , -0.64246983E-01 , 0.5E-06 , "X08Y0" );
  cs_probe_set_add_probe(xset_0 , -0.241956E-01 , -0.64246983E-01 , 0.5E-06 , "X09Y0" );
  cs_probe_set_add_probe(xset_0 , -0.227292E-01 , -0.64246983E-01 , 0.5E-06 , "X10Y0" );
  cs_probe_set_add_probe(xset_0 , -0.212628E-01 , -0.64246983E-01 , 0.5E-06 , "X11Y0" );
  cs_probe_set_add_probe(xset_0 , -0.197964E-01 , -0.64246983E-01 , 0.5E-06 , "X12Y0" );
  cs_probe_set_add_probe(xset_0 , -0.183300E-01 , -0.64246983E-01 , 0.5E-06 , "X13Y0" );
  cs_probe_set_add_probe(xset_0 , -0.168636E-01 , -0.64246983E-01 , 0.5E-06 , "X14Y0" );
  cs_probe_set_add_probe(xset_0 , -0.153972E-01 , -0.64246983E-01 , 0.5E-06 , "X15Y0" );
  cs_probe_set_add_probe(xset_0 , -0.139308E-01 , -0.64246983E-01 , 0.5E-06 , "X16Y0" );
  cs_probe_set_add_probe(xset_0 , -0.124644E-01 , -0.64246983E-01 , 0.5E-06 , "X17Y0" );
  cs_probe_set_add_probe(xset_0 , -0.109980E-01 , -0.64246983E-01 , 0.5E-06 , "X18Y0" );
  cs_probe_set_add_probe(xset_0 , -0.953160E-02 , -0.64246983E-01 , 0.5E-06 , "X19Y0" );
  cs_probe_set_add_probe(xset_0 , -0.806520E-02 , -0.64246983E-01 , 0.5E-06 , "X20Y0" );
  cs_probe_set_add_probe(xset_0 , -0.659880E-02 , -0.64246983E-01 , 0.5E-06 , "X21Y0" );
  cs_probe_set_add_probe(xset_0 , -0.513240E-02 , -0.64246983E-01 , 0.5E-06 , "X22Y0" );
  cs_probe_set_add_probe(xset_0 , -0.366600E-02 , -0.64246983E-01 , 0.5E-06 , "X23Y0" );
  cs_probe_set_add_probe(xset_0 , -0.219960E-02 , -0.64246983E-01 , 0.5E-06 , "X24Y0" );
  cs_probe_set_add_probe(xset_0 , -0.733200E-03 , -0.64246983E-01 , 0.5E-06 , "X25Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.733200E-03 , -0.64246983E-01 , 0.5E-06 , "X26Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.219960E-02 , -0.64246983E-01 , 0.5E-06 , "X27Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.366600E-02 , -0.64246983E-01 , 0.5E-06 , "X28Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.513240E-02 , -0.64246983E-01 , 0.5E-06 , "X29Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.659880E-02 , -0.64246983E-01 , 0.5E-06 , "X30Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.806520E-02 , -0.64246983E-01 , 0.5E-06 , "X31Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.953160E-02 , -0.64246983E-01 , 0.5E-06 , "X32Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.109980E-01 , -0.64246983E-01 , 0.5E-06 , "X33Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.124644E-01 , -0.64246983E-01 , 0.5E-06 , "X34Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.139308E-01 , -0.64246983E-01 , 0.5E-06 , "X35Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.153972E-01 , -0.64246983E-01 , 0.5E-06 , "X36Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.168636E-01 , -0.64246983E-01 , 0.5E-06 , "X37Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.183300E-01 , -0.64246983E-01 , 0.5E-06 , "X38Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.197964E-01 , -0.64246983E-01 , 0.5E-06 , "X39Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.212628E-01 , -0.64246983E-01 , 0.5E-06 , "X40Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.227292E-01 , -0.64246983E-01 , 0.5E-06 , "X41Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.241956E-01 , -0.64246983E-01 , 0.5E-06 , "X42Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.256620E-01 , -0.64246983E-01 , 0.5E-06 , "X43Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.271284E-01 , -0.64246983E-01 , 0.5E-06 , "X44Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.285948E-01 , -0.64246983E-01 , 0.5E-06 , "X45Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.300612E-01 , -0.64246983E-01 , 0.5E-06 , "X46Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.315276E-01 , -0.64246983E-01 , 0.5E-06 , "X47Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.329940E-01 , -0.64246983E-01 , 0.5E-06 , "X48Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.344604E-01 , -0.64246983E-01 , 0.5E-06 , "X49Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.359268E-01 , -0.64246983E-01 , 0.5E-06 , "X50Y0" );
  cs_probe_set_add_probe(xset_0 ,  0.366600E-01 , -0.64246983E-01 , 0.5E-06 , "X51Y0" );

  /* ********************************************************************************** */

  /* ################################################################################ */
  /* ################################################################################ */

  }
  

}
/*#endif*/

/*----------------------------------------------------------------------------*/
/*!
 * Override default frequency or calculation end based output.
 *
 * This allows fine-grained control of activation or deactivation,
 *
 * \param  nt_max_abs  maximum time step number
 * \param  nt_cur_abs  current time step number
 * \param  t_cur_abs   absolute time at the current time step
 */
/*----------------------------------------------------------------------------*/

#if 0
void
cs_user_postprocess_activate(int     nt_max_abs,
                             int     nt_cur_abs,
                             double  t_cur_abs)
{
  /* Use the cs_post_activate_writer() function to force the
   * "active" or "inactive" flag for a specific writer or for all
   * writers for the current time step.

   * the parameters for cs_post_activate_writer() are:
   *   writer_id <-- writer id, or 0 for all writers
   *   activate  <-- false to deactivate, true to activate */
   
  #if 0
  /*! [post_activate] */
  int writer_id = 0; /* 0: all writers */
  cs_post_activate_writer(writer_id, true);
  /*! [post_activate] */
  #endif
  
    /*! [post_activate] */
  if (nt_max_abs < 100) {
    int writer_id = 0; /* 0: all writers */
    cs_post_activate_writer(writer_id, true);
  }
  /*! [post_activate] */
  
}
#endif

/*----------------------------------------------------------------------------*/

END_C_DECLS
